import os, sys, copy

import scipy

import libtbx.phil
import libtbx.easy_pickle

import iotbx.pdb

master_phil = libtbx.phil.parse("""
direction = toref *fromref
    .type = choice
file
{
    input  = None
        .type = path
    output = None
        .type = path
}
dataset_pickle = ./pickles/dataset.pickle
    .type = path
overwrite = False
    .type = bool
verbose = False
    .type = bool
""")

def run(params):

    if params.dataset_pickle:
        assert os.path.exists(params.dataset_pickle), 'PICKLE DOES NOT EXIST: {!s}'.format(params.dataset_pickle)
    else:
        # Find the dataset pickle
        raise Exception('PLEASE PROVIDE DATASET PICKLE')

    if params.file.input:
        assert os.path.exists(params.file.input), 'FILE DOES NOT EXIST: {!s}'.format(params.file.input)
    else:
        # Find the dataset pickle
        raise Exception('PLEASE PROVIDE INPUT FILE')

    # Create output file if needed
    if not params.file.output:
        params.file.output = os.path.splitext(os.path.basename(params.file.input))[0] + '.{!s}.pdb'
        if params.direction == 'toref':   params.file.output = params.file.output.format('ref')
        else:                           params.file.output = params.file.output.format('native')
    if os.path.exists(params.file.output):
        if params.overwrite: os.remove(params.file.output)
        else: raise Exception('FILE ALREADY EXISTS: {!s}'.format(params.file.output))

    if params.verbose: print 'TRANSFORMING: {!s}'.format(params.file.input)
    if params.verbose: print 'USING DATASET FROM: {!s}'.format(params.dataset_pickle)
    if params.verbose: print 'WRITING FILE TO: {!s}'.format(params.file.output)

    # Load the dataset handler from the pickle
    d_handler = libtbx.easy_pickle.load(params.dataset_pickle)

    # Load the ligand to be twiddled
    hier = iotbx.pdb.hierarchy.input(params.file.input).hierarchy

    if d_handler.local_alignment_transforms:
        method = 'local'
        if params.direction == 'toref':
            map_hier = d_handler.new_structure().hierarchy
        else:
            map_hier = d_handler.new_structure().hierarchy
            map_hier.atoms().set_xyz(d_handler.transform_to_reference(points=map_hier.atoms().extract_xyz(), method=method))

        mappings = d_handler.find_nearest_calpha(   points = hier.atoms().extract_xyz(),
                                                    hierarchy = map_hier    )
    else:
        method = 'global'
        mappings = None

    if params.direction == 'toref':
        trans_points = d_handler.transform_to_reference(    points = hier.atoms().extract_xyz(),
                                                            method = method,
                                                            point_mappings = mappings   )
    elif params.direction == 'fromref':
        trans_points = d_handler.transform_from_reference(  points = hier.atoms().extract_xyz(),
                                                            method = method,
                                                            point_mappings = mappings   )

    # Try to add symmetry to the output file
    try:    crystal_symmetry = d_handler.mtz_summary.symmetry
    except: crystal_symmetry = None

    # Create new hierarchy and write out
    new_hier = hier.deep_copy()
    new_hier.atoms().set_xyz(trans_points)
    new_hier.write_pdb_file(    file_name        = params.file.output,
                                crystal_symmetry = crystal_symmetry     )

    return

#######################################

if __name__ == '__main__':
    from giant.jiffies import run_default
    run_default(run=run, master_phil=master_phil, args=sys.argv[1:], blank_arg_prepend=blank_arg_prepend)



import os

from bamboo.common.command import CommandManager
from bamboo.constants import DEFAULT_LIGAND_NAMES
from bamboo.wrappers.program_selector import allowed_builder_args

class BuilderObject(object):
    """Template Object for creating an Object to generate ligand restraints"""

    def __init__(self, time=True, verbose=True):

        self.allowed_args = allowed_builder_args
        self.ligname = DEFAULT_LIGAND_NAMES[0]
        # Settings
        self.time = time
        self.verbose = verbose
        self.runtime = -1.0
        self.timeout = 1800
        # Custom Init
        self._custom_init()

    def generate_ligand(self, ligsmile, outdir, outfile, flags=[]):
        """Generate ligand PDB and CIF from smile string"""

        # Prepare standard settings
        self._prepare_to_generate_ligand(ligsmile, outdir, outfile, flags)
        # Check to see if already run, and if not, run the custom part of the object - different for each program
        if os.path.exists(self.outpdb) and os.path.exists(self.outcif) and os.path.exists(self.outlog):
            if self.verbose:
                print('\tLigand already generated - DOING NOTHING')
        else:
            cmd_line_args, std_inpt_args = self._create_program_arguments(ligsmile, outdir, outfile, flags)
            # Initialise CommandManager
            self.builder = CommandManager(self.program)
            # Set Command-Line Args
            if cmd_line_args: self.builder.add_command_line_arguments(cmd_line_args)
            # Set Standard Input
            if std_inpt_args: self.builder.add_standard_input(std_inpt_args)
            # Set Parameters
            self.builder.set_timeout(timeout=self.timeout)
            # RUN
            self.builder.run()
            # Get runtime
            self.runtime = self.builder.runtime

            try:
                # Could add postprocess here
                pass

            finally:
                self.write_log_file()

        return self.outpdb, self.outcif, self.outlog

    def _prepare_to_generate_ligand(self, ligsmile, outdir, outfile, flags):
        """Set up the generic file names"""

        # Process outputfile
        if '/' in outfile:
            raise ValueError('outfile must be a file, not a path')
        # Record Template and Outdir
        self.outtemplate = os.path.join(outdir,outfile)
        self.outdir      = outdir
        # Record Filenames
        self.outpdb = self.outtemplate+'.pdb'
        self.outcif = self.outtemplate+'.cif'
        self.outlog = self.outtemplate+'.log'

    def write_log_file(self):
        """Write the log file"""

        with open(self.outlog,'w') as logfile:
            # Write out the input command
            logfile.write('\nCOMMAND\n\n')
            logfile.write('\n'.join(self.builder.cmd_line_args)+'\n')
            logfile.write('\nINPUT\n\n')
            logfile.write('\n'.join(self.builder.std_inp_lines)+'\n')
            # Write out & err
            logfile.write('\nSTDOUT\n\n')
            logfile.write(self.builder.output)
            logfile.write('\nSTDERR\n\n')
            logfile.write(self.builder.error)




import os, sys, copy, re, shutil

import libtbx.phil
import libtbx.easy_mp
from bamboo.common.command import CommandManager
from giant.xray.data import CrystalSummary

#######################################

blank_arg_prepend = {'.pdb':'input.pdb=', '.mtz':'input.mtz='}

master_phil = libtbx.phil.parse("""
input {
    pdb = None
        .type = path
    mtz = None
        .type = path
}
options {
    n_cycles = 0
        .type = int
    add_hydrogens = True
        .type = bool
    output_hydrogens = True
        .type = bool
    build_absent_atoms = True
        .type = bool
}
output {
    pdb = None
        .type = path
    mtz = None
        .type = path
    log = None
        .type = path
    suffix = '.refmac'
        .type = str
}
""")

#######################################

def run(params):

    in_pdb = params.input.pdb
    assert os.path.exists(in_pdb), 'PDB does not exist: {}'.format(in_pdb)
    in_mtz = params.input.mtz
    if in_mtz: assert os.path.exists(in_mtz), 'MTZ does not exist: {}'.format(in_mtz)

    if params.output.pdb: out_pdb = params.output.pdb
    else:                 out_pdb = os.path.splitext(in_pdb)[0] + params.output.suffix + '.pdb'

    if params.output.mtz: out_mtz = params.output.mtz
    elif in_mtz:          out_mtz = os.path.splitext(in_mtz)[0] + params.output.suffix + '.mtz'
    else:                 out_mtz = None

    if params.output.log: out_log = params.output.log
    else:                 out_log = os.path.splitext(in_pdb)[0] + params.output.suffix + '.log'

    ###########################################
    # COMMAND LINE COMMANDS
    ###########################################
    cm = CommandManager('refmac5')
    if in_pdb:  cm.add_command_line_arguments(['xyzin',in_pdb])
    if out_pdb: cm.add_command_line_arguments(['xyzout',out_pdb])
    if in_mtz:  cm.add_command_line_arguments(['hklin',in_mtz])
    if out_mtz: cm.add_command_line_arguments(['hklout',out_mtz])

    ###########################################
    # MAIN PARAMETERS
    ###########################################
    cm.add_standard_input('MAKE NCYC {}'.format(params.options.n_cycles))

    ###########################################
    # HYDROGENS
    ###########################################
    if params.options.add_hydrogens:
        cm.add_standard_input('MAKE HYDR A')
    else:
        cm.add_standard_input('MAKE HYDR Y')

    if params.options.output_hydrogens:
        cm.add_standard_input('MAKE HOUT Y')
    else:
        cm.add_standard_input('MAKE HOUT N')

    ###########################################
    # MISSING ATOMS
    ###########################################
    if params.options.build_absent_atoms:
        cm.add_standard_input('MAKE BUIL Y')
    else:
        cm.add_standard_input('MAKE BUIL N')

    ###########################################
    # LASTLY
    ###########################################
    cm.add_standard_input('END')

    ###########################################
    # RUN
    ###########################################
    print 'Running Refmac:'
    cm.print_settings()
    ret_code = cm.run()
    cm.write_output(log_file=out_log)
    if ret_code != 0:
        print '============================>'
        print 'Refmac returned with an error'
        print '============================>'
        print cm.output
        print '============================>'
        print cm.error
        print '============================>'

#######################################

if __name__=='__main__':
    from giant.jiffies import run_default
    run_default(run=run, master_phil=master_phil, args=sys.argv[1:], blank_arg_prepend=blank_arg_prepend)



import os, shutil, tempfile

from bamboo.common.command import CommandManager

def merge_cif_libraries(incifs, outcif):
    """Take a list of cifs and merge into one cif"""

    assert isinstance(incifs, list), "'incifs' is not a list!"
    assert len(incifs) > 1, "'incifs' must be two or more files!"

    current = incifs.pop(0)

    to_be_deleted = []

    for additional in incifs:

        # Create a file handle and path for the output
        temp_handle, temp_path = tempfile.mkstemp(suffix='.lib', prefix='libcheck_')
        to_be_deleted.append(temp_path)

        # Merge current and additional to temp_path
        LIBCHK = CommandManager('libcheck')
        LIBCHK.add_standard_input('_DOC N','_FILE_L {!s}'.format(current),'_FILE_L2 {!s}'.format(additional),'_FILE_O {!s}'.format(temp_path.replace('.lib','')),'_END')
        LIBCHK.run()

        current = temp_path

    shutil.copy(current, outcif)

    for file in to_be_deleted:
        os.remove(file)

    assert os.path.exists(outcif), 'OUTPUT CIF DOES NOT EXIST! {!s}'.format(outcif)

    return outcif





def run_custom_analyses(pandda):
    """If a list of hits is available, test to see whether the pandda identified them"""

    try:

        # ============================================================================>
        # ======================================>
        # Manual Analyses - These only need processing once
        # ======================================>
        # ============================================================================>
        analyses_dir = pandda.output_handler.get_dir('analyses')
        # ============================================================================>

        if 0:
            pandda.log('===================================>>>')
            pandda.log('Calculating Deviations of C-alphas between structures')

            rms = lambda vals: numpy.sqrt(numpy.mean(numpy.abs(vals)**2))
            norm = lambda vals: numpy.sqrt(numpy.sum(numpy.abs(vals)**2))

            # Pull all c-alpha sites for each structure
            all_sites = numpy.array([d.transform_points_to_reference(d.get_calpha_sites()) for d in pandda.datasets.all()])
            # Calculate the mean x,y,z for each c-alpha
            mean_sites = numpy.mean(all_sites, axis=0)
            # Differences from the mean
            diff_sites = all_sites - mean_sites
            # Euclidean norms of the distances moved
            diff_norms = numpy.apply_along_axis(norm, axis=2, arr=diff_sites)

            with open(os.path.join(analyses_dir,'calpha_variation.csv'), 'w') as fh:
                for row in diff_norms:
                    out_list = row.round(3).tolist()
                    out_line = ', '.join(map(str,out_list)) + '\n'
                    fh.write(out_line)

            pandda.log('Largest deviation from the mean site: {!s}'.format(diff_norms.max()))
            pandda.log('Average deviation from the mean site: {!s}'.format(diff_norms.mean()))

        # ============================================================================>

        if 0:
            pandda.log('===================================>>>')
            pandda.log('Clustering the Refined Structures')

            distance_matrix = []
            for d1 in pandda.datasets.all():
               distance_matrix.append([d1.transform_points_to_reference(d1.get_calpha_sites()).rms_difference(d2.transform_points_to_reference(d2.get_calpha_sites())) for d2 in pandda.datasets.all()])

            distance_matrix = numpy.array(distance_matrix)

            with open(os.path.join(analyses_dir,'calpha_distance_matrix.csv'), 'w') as fh:
                for row in distance_matrix:
                    out_list = row.round(3).tolist()
                    out_line = ', '.join(map(str,out_list)) + '\n'
                    fh.write(out_line)

        # ============================================================================>

    except:
        pandda.log('FAILURE DURING CUSTOM ANALYSES', True)
        raise

    return 0

# XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
# TODO REIMPLEMENT THIS IF IT CAN BE DONE SO THAT IT LOOKS NICE
#
#                # Image Blobs in the datasets using ccp4mg
#                if d_handler.hit_clusters:
#
#                    pandda.log('===================================>>>', True)
#                    pandda.log('Imaging blobs in Dataset {!s}'.format(d_handler.tag), True)
#
#                    sorted_blob_indices = d_handler.hit_clusters.sort(sorting_function=max)
#                    blob_num = len(sorted_blob_indices)
#
#                    for blob_rank, blob_grid_peak in enumerate(d_handler.hit_clusters.get_centroids(indices=sorted_blob_indices)):
#
#                        # Only produce a certain number of images
#                        if blob_rank == pandda.params.blob_search.blobs_to_image:
#                            break
#
#                        status_bar(n=blob_rank, n_max=blob_num)
#
#                        blob_cart_peak = [g*pandda.reference_grid().grid_spacing() for g in blob_grid_peak]
#
#                        # Make images of the blob
#                        pandda.image_blob(  script    = d_handler.output_handler.get_file('ccp4mg_script'),
#                                            image     = d_handler.output_handler.get_file('ccp4mg_png'),
#                                            d_handler = d_handler,
#                                            point_no  = blob_rank+1,
#                                            point     = blob_cart_peak,
#                                            towards   = centre_of_mass
#                                        )
#
#                    status_bar(n=blob_num, n_max=blob_num)
# XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX




# Adapted from Code by Sebastian Kelm

import os, sys, tempfile
import pandas

from bamboo.common.command import CommandManager
from bamboo.utils.mtz import MtzSummary

class Edstats(object):
    def __init__(self, mtz_file, pdb_file):
        scores, command = score_with_edstats_to_dict(mtz_file=mtz_file, pdb_file=pdb_file)
        self.scores = pandas.DataFrame.from_dict(scores)
        self._command = command

class EdstatsLogSummary:
    """Class to process, store and present the output from edstats.pl"""

    def __init__(self, logtext):
        self.logtext = logtext
        self.mean_biso = None
        self.rms_scores = {'atm_all_ZD+':None,'atm_all_ZD-':None,'atm_all_ZO':None,'atm_main_ZD+':None,'atm_main_ZD-':None,'atm_main_ZO':None,'atm_side_ZD+':None,'atm_side_ZD-':None,'atm_side_ZO':None,'het_all_ZD+':None,'het_all_ZD-':None,'het_all_ZO':None,'het_main_ZD+':None,'het_main_ZD-':None,'het_main_ZO':None,'het_side_ZD+':None,'het_side_ZD-':None,'het_side_ZO':None}

        # Populate fields
        self._parselogtext()

    def _parselogtext(self):
        """Read and sort through log input"""

        output = [line for line in self.logtext.split('\n') if line]
        for i, line in enumerate(output):
            if line.startswith('Overall mean Biso:'):
                self.mean_biso = float(line.replace('Overall mean Biso:',''))
            elif line.startswith('RMS Z-scores for protein residues:'):
                assert ['Main','Side','All'] == output[i+1].strip().split(), 'COLUMN HEADINGS ARE NOT WHAT IS EXPECTED! {!s}'.format(line)
                for incr in [2,3,4]:
                    score, main_val, side_val, all_val = output[i+incr].strip().split()
                    self.rms_scores['_'.join(['atm','main',score])] = float(main_val)
                    self.rms_scores['_'.join(['atm','side',score])] = float(side_val)
                    self.rms_scores['_'.join(['atm','all',score])] = float(all_val)
            elif line.startswith('RMS Z-scores for hetero residues:'):
                assert ['Main','Side','All'] == output[i+1].strip().split(), 'COLUMN HEADINGS ARE NOT WHAT IS EXPECTED! {!s}'.format(line)
                for incr in [2,3,4]:
                    score, main_val, side_val, all_val = output[i+incr].strip().split()
                    self.rms_scores['_'.join(['het','main',score])] = float(main_val)
                    self.rms_scores['_'.join(['het','side',score])] = float(side_val)
                    self.rms_scores['_'.join(['het','all',score])] = float(all_val)

########################################################################################################

def score_file_with_edstats(mtz_file, pdb_file):
    """Score pdb file against electron density"""

    # Score the complex with Edstats
    edstats = Edstats(mtz_file=mtz_file, pdb_file=pdb_file)
    if edstats.scores.empty and edstats._command.error:
        raise Exception('EDSTATS ERROR : {!s}'.format(edstats._command.error))
    # Process the std out of the program
    summary = EdstatsLogSummary(edstats._command.output)

    return edstats, summary

def score_with_edstats_to_dict(mtz_file, pdb_file):
    """Scores residues against density, then converts to dict"""

    # Generate the list of the EDSTATS scores for each residue
    scores, header, command = score_with_edstats_to_list(mtz_file, pdb_file)
    # Create dict for the mapping between residue_id and scores
    mapping = {}
    for label, values in scores:
        hash = {}
        for k,v in zip(header,values):
            hash[k] = v
        mapping[label] = hash

    return mapping, command

def score_with_edstats_to_list(mtz_file, pdb_file):
    """Scores residues against density, then returns list"""

    assert os.path.exists(mtz_file), 'MTZ FILE FOR EDSTATS DOES NOT EXIST! {!s}'.format(mtz_file)
    assert os.path.exists(pdb_file), 'PDB FILE FOR EDSTATS DOES NOT EXIST! {!s}'.format(mtz_file)

    # Create a file handle and path for the output
    temp_handle, temp_path = tempfile.mkstemp(suffix='.table', prefix='edstats_')

    # Find the labels in the mtzfile
    file_obj = MtzSummary(mtz_file)
    f_label = file_obj.label.f
    if not f_label:
        raise ReflectionException('MTZ Summary ERROR: No F Label Found in MTZ File: {!s}'.format(mtz_file))

    # Run EDSTATS on the files
    try:
        # Initialise Command Manager to run edstats
        command = CommandManager('edstats.pl')
        command.add_command_line_arguments(['-hklin',mtz_file,'-xyzin',pdb_file,'-output',temp_path,'-noerror','-flabel',f_label])
        command.set_timeout(timeout=600)
        command.run()
        # Read the output
        with os.fdopen(temp_handle) as f:
            output = f.read().strip().replace('\r\n','\n').replace('\r','\n').splitlines()
        command.file_output = output
    finally:
        os.remove(temp_path)

    # Process the output header
    if output:
        header = output.pop(0).split()
        assert header[:3] == ['RT', 'CI', 'RN'], 'EDSTATS OUTPUT HEADERS ARE NOT AS EXPECTED! {!s}'.format(output)
        num_fields = len(header)
        header = header[3:]
    else:
        header = []

    # List to be returned
    outputdata = []

    # Process the rest of the data
    for line in output:
        line = line.strip()
        if not line:
            continue

        fields = line.split()
        if len(fields) != num_fields:
            raise ValueError("Error Parsing EDSTATS output: Header & Data rows have different numbers of fields")

        # Get and process the residue information
        residue, chain, resnum = fields[:3]
        try:
            resnum = int(resnum)
            inscode = ' '
        except ValueError:
            inscode = resnum[-1:]
            resnum = int(resnum[:-1])

        # Remove the processed columns
        fields = fields[3:]

        # Process the other columns (changing n/a to None and value to int)
        for i, x in enumerate(fields):
            if x == 'n/a':
                fields[i] = None
            else:
                try:
                    fields[i] = int(x)
                except ValueError:
                    fields[i] = float(x)

        outputdata.append([(residue, chain, resnum, inscode),fields])

    return outputdata, header, command





import os, sys, glob, shutil
import wget

def get_pdb_index(filename='pdb_entry_type.txt'):
    if os.path.exists(filename): return filename
    tempfile = wget.download('ftp://ftp.wwpdb.org/pub/pdb/derived_data/pdb_entry_type.txt')
    if tempfile != filename: shutil.move(tempfile, filename)
    assert os.path.exists(filename)
    return filename

def download_pdb(pdb_id, filename):
    """Download pdb file to a filename"""
    if os.path.exists(filename): return filename
    try:    tempfile = wget.download('http://www.rcsb.org/pdb/files/{}.pdb'.format(pdb_id))
    except: print('Failed to download:{}'.format(pdb_id)); raise
    if tempfile != filename: shutil.move(tempfile, filename)
    assert os.path.exists(filename)
    return filename

def download_and_organise_pdb_as_necessary(pdb_id, out_dir='pdb-download'):
    """Download and organise the pdb file for a pdb code"""
    # Check the top directory
    if not os.path.exists(out_dir):
        try:    os.mkdir(out_dir)
        except: pass
    # Create subdirectory for the pdb file
    pdb_dir = os.path.join(out_dir, pdb_id[0])
    if not os.path.exists(pdb_dir):
        try:    os.mkdir(pdb_dir)
        except: pass
    # Get the pdb file
    pdb_file = os.path.join(pdb_dir, pdb_id+'.pdb')
    # Download file if necessary
    if not os.path.exists(pdb_file): download_pdb(pdb_id=pdb_id, filename=pdb_file)
    return pdb_file



import os, shutil

from bamboo.common.command import CommandManager
from bamboo.constants import DEFAULT_OUTPUT_CHAIN
from bamboo.xray.cryst import create_cryst_line_from_mtz
from bamboo.wrappers.program_selector import allowed_fitter_args
from bamboo.wrappers.symutils import map_to_reference_using_symmetry
from bamboo.wrappers.pdbutils import merge_pdb_files, change_residue_chain_and_number

class FitterObject(object):

    def __init__(self, time=True, verbose=True):

        self.allowedArgs = allowed_fitter_args
        self.preferChain = DEFAULT_OUTPUT_CHAIN[0]
        # Settings
        self.time = time
        self.verbose = verbose
        self.runtime = -1.0
        self.timeout = 99999
        # Custom Init
        self._custom_init()

    def fit_ligand(self, ligcif, ligpdb, mtz, apopdb, outdir, outfile, flags=[]):
        """Fit Ligand to a structure"""

        # Prepare input and output filenames
        self._prepare_to_fit_ligand(ligcif, ligpdb, mtz, apopdb, outdir, outfile, flags)

        # Check to see if already run, and if not, run the custom part of the object - different for each program
        if os.path.exists(self.outpdb) and os.path.exists(self.outlog):
            if self.verbose:
                print('\tLigand already fitted - DOING NOTHING')
            self._post_process(ligcif, ligpdb, mtz, apopdb, outdir, outfile, flags)
        else:
            if self.verbose:
                print('\tFitting using {!s}.'.format(self.name))
            cmd_line_args, std_inpt_args = self._create_program_arguments(ligcif, ligpdb, mtz, apopdb, outdir, outfile, flags)
            # Initialise CommandManager
            self.Fitter = CommandManager(self.program)
            # Set Command-Line Args
            if cmd_line_args: self.Fitter.SetArguments(cmd_line_args)
            # Set Standard Input
            if std_inpt_args: self.Fitter.SetInput(std_inpt_args)
            # Set Parameters
            self.Fitter.SetParameters(timeout=self.timeout)
            # RUN
            self.Fitter.Run()
            # Calculate runtime (seconds)
            self.runtime = self.Fitter.runtime

            try:
                # Process results and generate the list of modelled ligands
                if self.verbose:
                    print('\tFinished fitting - post-processing models.')
                self._post_process(ligcif, ligpdb, mtz, apopdb, outdir, outfile, flags)

                if not self.outmodels:
                    raise FittingError('No fitted models found! Log: {!s}'.format(self.outlog))

                if not os.path.exists(self.outpdb):
                    # Map the model of the best ligand to the asu and check it's near to the protein
                    self._map_to_asu(ligcif, ligpdb, mtz, apopdb, outdir, outfile, flags)
                    # Create symbolic links to the output file
                    self._link_output_files()

                # Merge the best ligand with the apo structure
                if not os.path.exists(self.mergedpdb):
                    self.Merger = merge_pdb_files(apopdb, self.outpdb, self.mergedpdb)

            finally:
                self.write_log_file()

        return self.outpdb, self.mergedpdb, self.outlog

    def _prepare_to_fit_ligand(self, ligcif, ligpdb, mtz, apopdb, outdir, outfile, flags):
        """Set up the generic file names"""

        # Process outputfile
        if '/' in outfile:
            raise ValueError('outfile must be a file, not a path')
        # Record Template and Outdir
        self.outtemplate = os.path.join(outdir,outfile)
        self.outdir      = outdir
        # Record Filenames
        self.outpdb      = self.outtemplate+'.lig.pdb'
        self.mergedpdb   = self.outtemplate+'.pdb'
        self.outlog      = self.outtemplate+'.log'
        # Storage
        self.outmodels   = []
        self.filtmodels  = []
        self.bestmodel   = None
        self.fittingdata = None

        # Prepare custom settings
        self._custom_prepare_to_fit_ligand(ligcif, ligpdb, mtz, apopdb, outdir, outfile, flags)

        return

    def _map_to_asu(self, ligcif, ligpdb, mtz, apopdb, outdir, outfile, flags):
        """Map the best ligand file to the asu of the protein - pick the symmetry version with the most contacts"""

        candidate = None

        if self.verbose:
            print('\tMapping fitted models back to the asymmetric unit.')

        for model in self.outmodels:

            ligand_sym_file = model.replace('.pdb','.sym.pdb')

            if os.path.exists(ligand_sym_file):
                self.filtmodels.append(model)
            else:
                try:
                    # Create Cryst Line
                    shutil.move(model, model+'.temp.pdb')
                    PDBSET = create_cryst_line_from_mtz(model+'.temp.pdb', ligand_sym_file, mtz)
                    # Rename the original file, then create a symmetry equivalent ligand that is close to the protein
#                    shutil.move(model, ligand_sym_file)
#                    candidate, ordered_files = map_structure_to_asu(apopdb, mtz, ligand_sym_file, model, delete_structures=True)
                    candidate = map_to_reference_using_symmetry(refpdb=apopdb, movpdb=ligand_sym_file, pdbout=model)
                    self.filtmodels.append(candidate)
                except (DistanceError, IOError) as err:
                    print(err)
                    continue

            # If it gets here we have a candidate
            # Break here for fast (1 model per fitter max)
#            break

        if not self.filtmodels:
            raise FittingError('No fitted models within range of protein!')
        else:
            # Rename all of the filtered models
            [change_residue_chain_and_number(filtfile) for filtfile in self.filtmodels]
            # Pick the `best` one to be the output
            self.bestmodel = self.filtmodels[0]

        return

    def _link_output_files(self):
        """Make links to the output files"""

        # Create Symlinks
        os.symlink(os.path.relpath(self.bestmodel,start=self.outdir),self.outpdb)

        return

    def write_log_file(self):
        """Write the log file"""

        if self.verbose:
            print('\tWriting fitting logfile.')

        with open(self.outlog,'w') as logfile:

            if hasattr(self, 'Fitter'):
                # Write out the input command
                logfile.write('\nCOMMAND\n\n')
                logfile.write('\n'.join(self.Fitter.command))
                logfile.write('\nINPUT\n\n')
                logfile.write(self.Fitter.inp)
                # Write out & err
                logfile.write('\nFITTER STDOUT\n\n')
                logfile.write(self.Fitter.out)
                logfile.write('\nFITTER STDERR\n\n')
                logfile.write(self.Fitter.err)

            if hasattr(self, 'Merger'):
                # Write out the input command
                logfile.write('\nCOMMAND\n\n')
                logfile.write('\n'.join(self.Merger.command))
                logfile.write('\nINPUT\n\n')
                logfile.write(self.Merger.inp)
                # Write out & err
                logfile.write('\nMERGER STDOUT\n\n')
                logfile.write(self.Merger.out)
                logfile.write('\nMERGER STDERR\n\n')
                logfile.write(self.Merger.err)

        return



from argparse import ArgumentParser

class ArgParser(ArgumentParser):
    """Minor Modifications to Argument Parser"""
    def parse_arguments(self, args=None):
        """Parse Arguments and return results also as a dictionary"""
        if args:
            self.inputs = self.parse_args(args)
        else:
            self.inputs = self.parse_args()

        return self.inputs, self.inputs.__dict__




from matplotlib import pyplot
pyplot.style.use('ggplot')

import numpy

def multiple_bar_plot(f_name, plot_vals, colour_bool=None, colour_vals=None):
    """Plot set of bar graphs in one figure"""
    if colour_bool:
        assert not colour_vals, 'Provide either colour_bool or colours'
        assert len(plot_vals) == len(colour_bool)
        assert map(len,plot_vals) == map(len,colour_bool)
    elif colour_vals:
        assert len(plot_vals) == len(colour_vals)
        assert map(len,plot_vals) == map(len,colour_vals)
    num_sites = len(plot_vals)
    fig, all_axes = pyplot.subplots(num_sites, sharex=True)
    # Change to list if only one subplot (pyplot...)
    if num_sites == 1: all_axes = [all_axes]
    # Iterate through different graphs
    for i_site, bar_vals in enumerate(plot_vals):
        num_vals = len(bar_vals)
        # Left sides of the bars
        bar_left = [x+0.6 for x in range(num_vals)]
        bar_hght = bar_vals
        # Colour of the bars
        if colour_bool:   bar_colr = ['limegreen' if b else 'red' for b in colour_bool[i_site]]
        elif colour_vals: bar_colr = colour_vals[i_site]
        else:             bar_colr = ['blue']*num_vals
        # Make the bar
        s_axis = all_axes[i_site]
        bar_bar = s_axis.bar(left=bar_left, height=bar_hght, width=0.8, color=bar_colr)
        s_axis.set_yticks([int(max(bar_hght)+0.5)])
    # Add xticks and save
    s_axis.set_xticks(range(1,max(map(len,plot_vals))+1))
    pyplot.tight_layout()
    pyplot.savefig(f_name)
    pyplot.close(fig)

def multiple_bar_plot_over_several_images(f_template, plot_vals, graphs_per_image=5, colour_bool=None, colour_vals=None):
    """Plot set of bar graphs in one figure"""
    if colour_bool:
        assert not colour_vals, 'Provide either colour_bool or colours'
        assert len(plot_vals) == len(colour_bool)
        assert map(len,plot_vals) == map(len,colour_bool)
    elif colour_vals:
        assert len(plot_vals) == len(colour_vals)
        assert map(len,plot_vals) == map(len,colour_vals)
    num_sites = len(plot_vals)
    # List of filenames to return
    filenames = []
    # Get the maximum x-axis
    max_x = max(map(len, plot_vals))
    # Iterate through different images
    for i_img in range(int(1.0*(num_sites-1)/graphs_per_image) + 1):
        # Create filename from template
        # Create filename from template
        f_name = f_template.format(i_img+1)
        filenames.append(f_name)
        # Get new image
        fig, all_axes = pyplot.subplots(graphs_per_image, sharex=False)
        # Change to list if only one subplot (pyplot...)
        if graphs_per_image == 1: all_axes = [all_axes]
        # Iterate through different graphs
        for i_plot, i_site in enumerate(range(graphs_per_image*i_img, graphs_per_image*i_img+graphs_per_image)):
            # Get the axis for this plot
            s_axis = all_axes[i_plot]
            # Change X-axis
            s_axis.set_xticks([-100,1]+range(5,max_x,5)+[max_x])
            s_axis.set_xlim([0,max_x+2])
            # Past the end of the data?
            if i_site >= num_sites:
                s_axis.set_yticks([])
                continue
            # Bar heights
            bar_vals = plot_vals[i_site]
            num_vals = len(bar_vals)
            # Left sides of the bars
            bar_left = [x+0.6 for x in range(num_vals)]
            bar_hght = bar_vals
            # Colour of the bars
            if colour_bool:   bar_colr = ['limegreen' if b else 'red' for b in colour_bool[i_site]]
            elif colour_vals: bar_colr = colour_vals[i_site]
            else:             bar_colr = ['blue']*num_vals
            # Make the bar
            bar_bar = s_axis.bar(left=bar_left, height=bar_hght, width=0.8, color=bar_colr)
            if   colour_bool: s_axis.set_title('Site {}: {} hits'.format(i_site+1, sum(colour_bool[i_site])))
            elif colour_vals: s_axis.set_title('Site {}: {} hits'.format(i_site+1, colour_vals[i_site].count('limegreen')))
            else:             s_axis.set_title('Site {}'.format(i_site+1))
            # Change Y-axis
            s_axis.set_yticks([int(max(bar_hght)+0.5)])
        # Save
        pyplot.tight_layout()
        pyplot.savefig(f_name)
        pyplot.close(fig)
    return filenames



import os, sys

from bamboo.common.command import CommandManager
from bamboo.utils.mtz import MtzSummary

def convert_intensities_to_amplitudes(mtzin, mtzout):
    """Takes an input mtz and converts the intensities to structure factors for model building"""

    mtzobj = MtzSummary(mtzin)

    # Copy data columns from new mtz file
    I, SIGI = mtzobj.label.i, mtzobj.label.sigi
    if not (I and SIGI):
        raise LabelError('No Intensities found in {!s}'.format(mtzin))

    # TODO If rfree is present, retain rfree from the reference mtz
#    RFree = mtzobj.label.free

    # Initialise Commander
    CTRUNC = CommandManager('ctruncate')
    # Set command arguments
    CTRUNC.add_command_line_arguments('-mtzin', mtzin, '-mtzout', mtzout, '-colin', '/*/*/[{!s},{!s}]'.format(I,SIGI))
    # Run!
    CTRUNC.run()

    if not os.path.exists(mtzout):
        raise ExternalProgramError('CTRUNCATE has failed to convert intensities to SFs. {!s}\nOUT: {!s}\nERR: {!s}'.format(mtzin, CTRUNC.out, CTRUNC.err))

    return CTRUNC

def apply_rfree_set(refmtz, mtzin, mtzout):
    """Takes an input mtz and a reference mtz and transplants the rfree flags"""

    tempmtz = mtzin.replace('.mtz','.temp.mtz')

    refobj = MtzSummary(refmtz)
    newobj = MtzSummary(mtzin)

    # Copy data columns from new mtz file
    F1, SIGF1 = newobj.label.f, newobj.label.sigf
    if not (F1 and SIGF1):
        raise LabelError('No Amplitudes found in {!s}'.format(mtzin))

    # Get the spacegroup and the rfree from the reference mtz
    sgno = refobj.data.spacegroupno
    RFree2 = refobj.label.free
    if not sgno:
        raise LabelError('No Spacegroup information found in {!s}'.format(refmtz))
    if not RFree2:
        raise LabelError('No RFreeFlags found in {!s}'.format(refmtz))

    # Initialise Commander
    CAD = CommandManager('cad')
    # Set command arguments
    CAD.add_command_line_arguments('hklin1', mtzin, 'hklin2', refmtz, 'hklout', tempmtz)
    # Set inputs
    CAD.add_standard_input(['symmetry {!s}'.format(sgno),'labin file_number 1 E1={!s} E2={!s}'.format(F1, SIGF1), \
                                               'labout file_number 1 E1={!s} E2={!s}'.format(F1, SIGF1), \
                                               'labin file_number 2 E1={!s}'.format(RFree2), \
                                               'labout file_number 2 E1={!s}'.format(RFree2),'END'])
    # Run!
    CAD.run()

    if not os.path.exists(tempmtz):
        raise ExternalProgramError('CAD has failed to transplant RFree Flags. {!s}\nOUT: {!s}\nERR: {!s}'.format(mtzin, CAD.out, CAD.err))

    # Now complete the free r set

    # Initialise Commander
    FREE = CommandManager('freerflag')
    # Set command arguments
    FREE.add_command_line_arguments('hklin', tempmtz, 'hklout', mtzout)
    # Set inputs
    FREE.add_standard_input(['COMPLETE FREE={!s}'.format(RFree2), 'END'])
    # Run!
    FREE.run()

    if not os.path.exists(mtzout):
        raise ExternalProgramError('freerflag has failed to complete the RFree Flag set. {!s}\nOUT: {!s}\nERR: {s}'.format(tempmtz, CAD.out, CAD.err))

    os.remove(tempmtz)

    return CAD, FREE




import os, time, re

from bamboo.wrappers.ligand_fitters import FitterObject

class LigandfitObject(FitterObject):

    def _custom_init(self):
        self.program = 'phenix.ligandfit'
        self.name = 'ligandfit'

    def _custom_prepare_to_fit_ligand(self, ligcif, ligpdb, mtz, apopdb, outdir, outfile, flags):
        """Prepare custom settings before fitting the ligand"""

        # Set the ligandfit output directory
        self.workdir = os.path.join(outdir,'LigandFit_run_1_')
        # Check for existence of other LigandFit directories
        run_directory = 1
        # Increment output number until we get a new one (This will be where phenix puts the output)
        while os.path.exists(self.workdir):
            run_directory += 1
            self.workdir = os.path.join(outdir,'LigandFit_run_{!s}_'.format(run_directory))

        # Check the penultimate one. If there's a fitting summary, take that one instead (pipeline already run before)
        if (run_directory > 1) and os.path.exists(self.outpdb) and os.path.exists(self.outlog):
            test_workdir = os.path.join(outdir,'LigandFit_run_{!s}_'.format(run_directory-1))
            test_outscores = os.path.join(test_workdir,'LigandFit_summary.dat')

            if os.path.exists(test_outscores):
                self.workdir = test_workdir

        # Where the scores and output files from the program will be recorded
        self.outscores = os.path.join(self.workdir,'LigandFit_summary.dat')

        # Change directory because of the 'unique' way that phenix writes its files
        os.chdir(outdir)

    def _create_program_arguments(self, ligcif, ligpdb, mtz, apopdb, outdir, outfile, flags):
        """Create command line and standard input to fit the ligand"""

        # Format Settings
        cmd_line_flags, std_inpt_flags = self._convert_flags(flags)
        # Form Command Line Args
        cmd_line_args = ['model='+apopdb,'data='+mtz,'ligand='+ligpdb,'cif_def_file_list='+ligcif]
        cmd_line_args.extend(cmd_line_flags)
        # Form Standard Input
        std_inpt_args = []
        std_inpt_args.extend(std_inpt_flags)

        return cmd_line_args, std_inpt_args

    def _post_process(self, ligcif, ligpdb, mtz, apopdb, outdir, outfile, flags):

        # Get the output models
        self._find_output_files()

        return

    def _convert_flags(self, flags):
        """Take the flags passed and generate the command string appropriately"""

        cmd_line_flags = []
        std_inpt_flags = []

        cmd_line_flags.append('number_of_ligands=5')

        return cmd_line_flags, std_inpt_flags

    def _find_output_files(self):
        """Reads the LigandFit summary to find the output files"""

        workdir = self.workdir
        summary = self.outscores

        if not os.path.exists(summary):
            raise FittingError('No Fitting Summary File Found: {!s}\n\t\tLog: {!s}'.format(summary, self.outlog))

        summary_contents = open(summary,'r').readlines()

        fields = ['Path','RSCC','Ligandfit Score','Num Atoms']
        rows = []
        data = {}

        for i, line in enumerate(summary_contents):
            if line.startswith('SINGLE LIGAND MODELS'):
                # Extract lists
                files = eval(line.replace('SINGLE LIGAND MODELS: ',''))
                newline = summary_contents[i+1].strip()
                assert newline.startswith('SINGLE LIGAND CC VALUES: '), 'LIGANDFIT SCORE FILE NOT IN EXPECTED FORMAT! {!s}'.format(summary)
                rsccs = eval(newline.replace('SINGLE LIGAND CC VALUES: ',''))
                newline = summary_contents[i+2].strip()
                assert newline.startswith('SINGLE LIGAND SCORES   : '), 'LIGANDFIT SCORE FILE NOT IN EXPECTED FORMAT! {!s}'.format(summary)
                ligsc = eval(newline.replace('SINGLE LIGAND SCORES   : ',''))
                newline = summary_contents[i+3].strip()
                assert newline.startswith('SINGLE LIGAND PLACED   : '), 'LIGANDFIT SCORE FILE NOT IN EXPECTED FORMAT! {!s}'.format(summary)
                noats = eval(newline.replace('SINGLE LIGAND PLACED   : ',''))
                # zip into table
                rows = zip(files,rsccs,ligsc,noats)

        if not (fields and rows):
            raise FittingError('No Fitted Files Found in Summary File: {!s}\n\t\tLog: {!s}'.format(summary, self.outlog))

        # Order by Ligandfit score
        for i, row in enumerate(sorted(rows,key=lambda entr: float(entr[2]),reverse=True)):
            # Create a dictionary of the row information
            rowdict = dict(zip(fields,row))
            # Create a rank (the files are already ordered by a rhofit score)
            rank = i+1
            # Add a couple more fields to the dict
            rowdict['Rank'] = rank
            # Create a common output score
            rowdict['InternalScore'] = rowdict['Ligandfit Score']
            # Check path exists
            assert os.path.exists(rowdict['Path']), 'Ligandfit Output File does not exist: {!s}'.format(rowdict)

            # Add to list of output model data (ordered by rank from ligandfit)
            data[rowdict['Path']] = rowdict
            # Add path to list of output models (ordered by rank from ligandfit)
            self.outmodels.append(rowdict['Path'])

        # Store scoring data
        self.fittingdata = data

        return




import os, time

from bamboo.wrappers.refiners import RefinerObject

class RefmacObject(RefinerObject):

    def _custom_init(self):
        self.program = 'refmac5'
        self.name = 'refmac'

    def _create_program_arguments(self, inpdb, inmtz, outdir, outfile, incif, flags):
        """Run Refmac to refine the structure"""

        # Format Settings
        cmd_line_flags, std_inpt_flags = self._convert_flags(flags)
        # Form Command Line Args
        cmd_line_args = ['XYZIN',inpdb,'HKLIN',inmtz,'XYZOUT',self.outpdb,'HKLOUT',self.outmtz]
        # Form Std Input
        std_inpt_args = []
        # Add cif files
        if incif and isinstance(incif, str):
            cmd_line_args.extend(['LIBIN',incif,'LIBOUT',self.outcif])
        elif incif and isinstance(incif, list):
            cmd_line_args.extend(['LIBIN',incif[-1],'LIBOUT',self.outcif])
        # Add processed flags
        cmd_line_args.extend(cmd_line_flags)
        std_inpt_args.extend(std_inpt_flags)
        std_inpt_args.append('END')

        return cmd_line_args, std_inpt_args

    def _post_process(self, inpdb, inmtz, outdir, outfile, incif, flags):
        pass

    def _convert_flags(self, flags):
        """Take the flags passed and generate the command string appropriately"""

        cmd_line_flags = []
        std_inpt_flags = []

        if 'keepH' in flags:
            flags.remove('keepH')
            std_inpt_flags.append('HOUT YES')
        # Check for refine b-factors
        if 'bonly' in flags:
            flags.remove('bonly')
            std_inpt_flags.append('REFI BONLY')
        # Check for noexit command for new ligands
        if 'noexit' in flags:
            flags.remove('noexit')
            std_inpt_flags.append('MAKE NEWLigand Noexit')
        # Check for occupancy refinement
        occu_residues = [f[1] for f in flags if f[0] == 'occupancy']
        # Add occupancy commands
        if occu_residues:
            # Clear up flags
            [flags.remove(f) for f in flags if f[1] in occu_residues]
            # Generate Command list for occupancy refinement
            group = 1
            for label in occu_residues:
                chain = label[0]
                resid = label[1]
                inp = 'OCCU GROUP ID {!s} CHAIN {!s} RESIDUE {!s}'.format(group, chain, resid)
                std_inpt_flags.append(inp)
                group += 1
            std_inpt_flags.append('OCCU REFINE')

        return cmd_line_flags, std_inpt_flags




from scipy.optimize import fsolve
import numpy

import time

def variable_sigma_d_log_likelihood(est_sigma, est_mu, obs_vals, obs_error):
    """Calculate the value of the differentiated log likelihood for the values of mu, sigma"""
    term1 = (obs_vals - est_mu)**2 / ((est_sigma**2 + obs_error**2)**2)
    term2 = 1 / (est_sigma**2 + obs_error**2)
    return numpy.sum(term1) - numpy.sum(term2)

def estimate_true_underlying_sd(obs_vals, obs_error, est_mu=None, est_sigma=1e-16, try_number=1):
    """Given a  set of observations `obs_vals` with estimated errors `obs_sigma`, estimate the mean and sd of the underlying distribution"""

    if try_number > 1:
        #print 'ITERATION {!s} - EST SIGMA: {!s}'.format(try_number, est_sigma)
        if try_number > 10:
            raise Exception('TOO MANY ITERATIONS IN OSPINA FUNCTION')

    obs_vals  = numpy.array(obs_vals)
    obs_error = numpy.array(obs_error)

    if not est_mu:
        # Calculate the mean of the sample - this is a good estimator of the true mean
        est_mu = numpy.mean(obs_vals)

    # Estimate the sigma of the underlying distribution
    answer = abs(fsolve(func=variable_sigma_d_log_likelihood, x0=est_sigma, args=(est_mu, obs_vals, obs_error)))[0]

    if answer > 2e40:
        est_sigma = est_sigma/1000
        answer = estimate_true_underlying_sd(obs_vals=obs_vals, obs_error=obs_error, est_mu=est_mu, est_sigma=est_sigma, try_number=try_number+1)

    return answer

if __name__ == '__main__':

    # True values we are trying to estimate
    true_mean = 1
    true_sd   = 0.11

    # Number of observations
    num_obs = 200

    # Number of times to run
    num_cycles = 10000

    guesses  = []

    start_t = time.time()

    for attempt in xrange(num_cycles):

#        print('============================>')

        # Sample the original distribution
        true_vals = true_mean + true_sd*numpy.random.randn(num_obs)
#        print('MEAN OF TRUE VALS: {!s} ({!s})'.format(round(numpy.mean(true_vals),3), true_mean))
#        print(' STD OF TRUE VALS: {!s} ({!s})'.format(round(numpy.std(true_vals),3), true_sd))

        # Create a random selection of sigmas for the different observations
        obs_error = numpy.abs(0.45 + 0.01*numpy.random.randn(num_obs))
#        print('MEAN OF OBS ERROR: {!s} ({!s})'.format(round(numpy.mean(obs_error),3), '2ish'))
#        print(' STD OF OBS ERROR: {!s} ({!s})'.format(round(numpy.std(obs_error),3), '1ish'))

        # Noise to be added to the true observations
        obs_noise = numpy.random.randn(num_obs)
#        print('MEAN OF OBS NOISE: {!s} ({!s})'.format(round(numpy.mean(obs_noise),3), 0))
#        print(' STD OF OBS NOISE: {!s} ({!s})'.format(round(numpy.std(obs_noise),3), 1))

        # Create fake data!
        obs_vals = true_vals + obs_error * obs_noise
#        print(' MEAN OF OBS VALS: {!s}'.format(round(numpy.mean(obs_vals),3)))
#        print('  STD OF OBS VALS: {!s}'.format(round(numpy.std(obs_vals),3)))

        out = estimate_true_underlying_sd(obs_vals=obs_vals, obs_error=obs_error, est_sigma=0.1)

        print(' ESTIMATED VALUES: {!s}'.format(round(out,5)))

        guesses.append(out)

    print('============================>')

    end_t = time.time()
    print('    TIME TAKEN TOTAL: {!s} (Seconds)'.format(int(end_t - start_t)))
    print('TIME TAKEN PER CYCLE: {!s} (Millionths)'.format(int(1000000*(end_t-start_t)/num_cycles)))

    print('============================>')

    print('MEAN OF ESTIMATES: {!s}'.format(round(numpy.mean(guesses),5)))
    print(' STD OF ESTIMATES: {!s}'.format(round(numpy.std(guesses),5)))



import os, sys, copy, re, shutil

import libtbx.phil
import iotbx.mtz
from bamboo.common.command import CommandManager

#######################################

blank_arg_prepend = {'.mtz':'input.mtz=', '.pdb':'input.pdb=', None:'column.label='}

master_phil = libtbx.phil.parse("""
input {
    mtz = None
        .multiple = False
        .type = path
    pdb = None
        .multiple = False
        .type = path
}
options {
    column {
        label = 2FOFCWT,PH2FOFCWT
            .help = 'columns to be extracted from the input mtz files'
            .type = str
            .multiple = False
    }
}
output {
    suffix = '.with-F000.mtz'
        .type = str
}
""")

#######################################

def run(params):

    assert params.input.pdb and os.path.exists(params.input.pdb), 'Need to provide an pdb file'
    assert params.input.mtz and os.path.exists(params.input.mtz), 'Need to provide an mtz file'

    out_mtz = os.path.splitext(params.input.mtz)[0]+params.output.suffix
    assert not os.path.exists(out_mtz)
    out_log = out_mtz.replace('.mtz', '.log')

    ###########################################
    # Load diffraction data
    ###########################################
    diff_data = iotbx.mtz.object(params.input.mtz)

    ###########################################
    # COMMAND LINE COMMANDS
    ###########################################
    cm = CommandManager('phenix.f000')
    cm.add_command_line_arguments(params.input.pdb)

    ###########################################
    # RUN
    ###########################################
    print 'Running CAD:'
    cm.print_settings()
    ret_code = cm.run()
    cm.write_output(log_file=out_log)
    if 1 or (ret_code != 0):
        print '============================>'
        print 'phenix.f000 returned with an error'
        print '============================>'
        print cm.output
        print '============================>'
        print cm.error
        print '============================>'

    f000_regex = re.compile('Estimate of F\(0,0,0\)=(.*?) given mean bulk-solvent density (.*?) and fraction (.*?)\n')
    f000_details = f000_regex.findall(cm.output)

    assert len(f000_details) == 1, 'Error extracting f000 from output of phenix.f000'
    f000, solvent_density, solvent_fraction = map(float,f000_details[0])
    print 'F000 is {}'.format(f000)

    ###########################################
    # Append to diffraction data
    ###########################################

    col_labs = params.options.column.label.split(',')
    print 'Extracting column labels: {}'.format(col_labs)

    miller_dict = diff_data.as_miller_arrays_dict()
    ampl_data = [d for l,d in miller_dict.items() if l[2] in col_labs]
    assert ampl_data, 'Matching AMPLITUDE DATA not in mtz data: {}'.format(ampl)
    assert len(ampl_data) == 2
    ampl_data = ampl_data[0]
    assert ampl_data.info().labels == col_labs

    # Create new objects
    new_data = ampl_data.customized_copy()
    # Append the F000 term
    new_data.indices().append((0,0,0))
    new_data.data().append(f000)
    # Sort for kicks
    new_data = new_data.sort('packed_indices')

#    from IPython import embed; embed(); assert 0;

    # Create new "dataset" from the miller array of the amplitudes
    mtz_dataset = new_data.as_mtz_dataset(column_root_label="2FOFCWT-ABS")
    # Convert "dataset" to an "object"
    mtz_object = mtz_dataset.mtz_object()
    mtz_object.write(file_name = out_mtz)

#######################################

if __name__=='__main__':
    from giant.jiffies import run_default
    run_default(run=run, master_phil=master_phil, args=sys.argv[1:], blank_arg_prepend=blank_arg_prepend)



import numpy

from matplotlib import pyplot

from bamboo.maths import finite

def simple_histogram(filename, data, title, x_lab='x', n_bins=30):
    """Quick histogram function"""

    fig = pyplot.figure()
    pyplot.title(title)
    pyplot.hist(x=finite(data), bins=n_bins)
    pyplot.xlabel(x_lab)
    pyplot.ylabel('Count')
    pyplot.tight_layout()
    pyplot.savefig(filename)
    pyplot.close(fig)

    return

def simple_bar(filename, y_vals, x_labels, title, x_lab='x', y_lab='y', x_lim=None, y_lim=None):
    """Quick bar plot function"""

    assert len(y_vals) == len(x_labels)

    # Plot sequential bars
    x_vals = numpy.arange(len(y_vals))

    fig = pyplot.figure()
    pyplot.title(title)
    pyplot.bar(left=x_vals, height=y_vals, width=1)
    pyplot.xlabel(x_lab)
    pyplot.ylabel(y_lab)
    pyplot.xticks(x_vals+0.5, x_labels)
    pyplot.xlim(x_lim)
    pyplot.ylim(y_lim)
    pyplot.tight_layout()
    pyplot.savefig(filename)
    pyplot.close(fig)

    return

def simple_scatter(filename, x_vals, y_vals, title, x_lab='x', y_lab='y', x_lim=None, y_lim=None):
    """Quick scatter plot function"""

    assert len(x_vals) == len(y_vals)

    fig = pyplot.figure()
    pyplot.title(title)
    pyplot.scatter(x=x_vals, y=y_vals)
    pyplot.xlabel(x_lab)
    pyplot.ylabel(y_lab)
    pyplot.xlim(x_lim)
    pyplot.ylim(y_lim)
    pyplot.tight_layout()
    pyplot.savefig(filename)
    pyplot.close(fig)

    return



import os, sys, shutil

from rdkit import Chem

from bamboo.utils.mtz import get_mtz_summary_dict
from bamboo.wrappers.symutils import generate_symmetry_mates
from bamboo.rdkit_utils.contacts.utils import order_structures_by_minimum_distance_to_reference, order_structures_by_number_of_contacts_to_reference

def generate_full_unit_cell(pdbin, pdbout, mtz=None, sgno=None, cell=None):
    """Map pdbin to the whole of the unit cell using symmetry operations"""

    if not os.path.exists(pdbin):
        raise IOError('pdbin does not exist! {!s}'.format(pdbin))
    if not pdbout.endswith('.pdb'):
        pdbout += '.pdb'
    if os.path.exists(pdbout):
        raise Exception('File already exists: {!s}'.format(pdbout))

    if (sgno != None) and (cell != None):
        pass
    elif (mtz != None):
        # Get the spacegroup number from the mtzfile
        mtzsummary = get_mtz_summary_dict(mtz)
        sgno = mtzsummary['spacegroupno']
        cell = mtzsummary['cell']
    else:
        raise Exception('mtz, or sgno AND cell, must be given!')

    # Generate the symmetry equivalent molecules
    pdbout = generate_symmetry_mates(pdbin=pdbin, pdbout=pdbout, sgno=sgno, cell=cell)

    return pdbout

def generate_symmetry_equivalent_ligands(pdbin, pdbouttemplate, mtz=None, sgno=None, cell=None, delete_structures=True):
    """Takes the ligand and a spacegroup number and creates symmetry copies. Returns a list of the new files
        Only designed to be used on ligands - do not use for large structures!"""

    if not os.path.exists(pdbin):
        raise IOError('pdbin does not exist! {!s}'.format(pdbin))
    if pdbouttemplate.endswith('.pdb'):
        pdbouttemplate = pdbouttemplate.replace('.pdb','')
    pdbout_individual = pdbouttemplate+'.{!s}.pdb'

    if (sgno != None) and (cell != None):
        pass
    elif (mtz != None):
        # Get the spacegroup number from the mtzfile
        mtzsummary = get_mtz_summary_dict(mtz)
        sgno = mtzsummary['spacegroupno']
        cell = mtzsummary['cell']
    else:
        raise Exception('SGNO and Cell, or MTZ must be given!')

    # Check that it can be read
    test_mol = Chem.MolFromPDBFile(pdbin)

    if not test_mol:
        raise RDkitReadError('"generate_symmetry_equivalent_ligands": Could not read input molecule {!s}'.format(pdbin))

    # Generate the symmetry equivalent molecules
    pdbtemp = generate_symmetry_mates(pdbin=pdbin, pdbout=pdbouttemplate+'.allsym.pdb', sgno=sgno, cell=cell)

    if not os.path.exists(pdbtemp):
        raise IOError('No Symmetry mates have been generated for {!s}'.format(pdbin))

    # Now use rdkit to split them into separate molecules
    all_mols = Chem.MolFromPDBFile(pdbtemp)

    # This will fail if the model is on an axis of symmetry (molecule maps on top of itself) - return the original model
    if not all_mols:
        # Copy the input model to the output
        out_filename = pdbout_individual.format(0)
        shutil.copy(pdbin, out_filename)
        # Delete the temporary file and return
        if delete_structures:
            os.remove(pdbtemp)
        return [out_filename]

    ind_mols = Chem.GetMolFrags(all_mols, asMols=True)

    # Write them to individual files
    outfiles = []
    for i, mol in enumerate(ind_mols):
        out_filename = pdbout_individual.format(i)
        Chem.MolToPDBFile(mol, out_filename)
        outfiles.append(out_filename)

    # Remove the tempfile
    if delete_structures: os.remove(pdbtemp)

    return outfiles




import os, shutil, tempfile, tarfile

import bamboo.misc

from bamboo.common.command import CommandManager

def run_xia2(img_dir, out_dir, method='dials', verbose=True):
    """Run xia2 on the images in img_dir, and put the results in out_dir"""

    assert method in ['2d','3d','3dii','dials'], 'Invalid method provided'

    assert os.path.exists(img_dir), 'Image directory does not exist'
    if not os.path.exists(out_dir): os.mkdir(out_dir)

    # Convert to absolute path to allow folder changing
    img_dir = os.path.abspath(img_dir)
    out_dir = os.path.abspath(out_dir)

    # Store current directory
    cur_dir = os.getcwd()

    try:

        # Output logfile
        xia2_log = os.path.join(out_dir, 'xia2.log')

        # Got to directory to run MLFSOM
        os.chdir(out_dir)

        xia2 = CommandManager('xia2')
        xia2.add_command_line_arguments('-'+method, img_dir)
        if verbose: print('Running xia2 on {}').format(img_dir)
        rc = xia2.run()
        if rc != 0:
            print('STDOUT:')
            print(xia2.output)
            print('STDERR:')
            print(xia2.error)
        with open(xia2_log, 'a') as fh:
            fh.write('==================================================>\n')
            fh.write('xia2 log\n')
            fh.write('==================================================>\n')
            fh.write(xia2.output+'\n')
            fh.write('==================================================>\n')
            fh.write('xia2 errors\n')
            fh.write('==================================================>\n')
            fh.write(xia2.error+'\n')
            fh.write('==================================================>\n')

    finally:
        # Change back to original directory
        os.chdir(cur_dir)

    return xia2

def run_mlfsom(pdb_file, res_h, out_dir='mlfsom_out', n_images=180, osc=1.0, energy=12660, verbose=True):
    """Run MLFSOM on the given file."""

    ftd_pre = ['1H87.pdb', 'example.com', 'example.mat', 'mlfsom.log', 'refined.pdb', 'pristine.mtz']

    assert not os.path.exists(out_dir), 'output directory already exists!'
    assert os.path.exists(pdb_file)
    assert isinstance(n_images, int)

    # Store current directory
    cur_dir = os.getcwd()

    try:
        # Create a temporary directory for mlfsom to run in (and go to it)
        tmp_dir = tempfile.mkdtemp(prefix='mlfsom_')
        mlfsom_pdb = os.path.join(tmp_dir, 'refined.pdb')
        mlfsom_mtz = os.path.join(tmp_dir, 'pristine.mtz')
        mlfsom_log = os.path.join(tmp_dir, 'mlfsom.log')
        mlfsom_img = 'image_###.img'

        # Get the mlfsom tarball and move it to the temporary directory
        mlfsom_file = os.path.join(bamboo.misc.__path__[0], 'mlfsom.tar.gz')
        assert os.path.exists(mlfsom_file), 'MLFSOM tarball not found'
        shutil.copy(mlfsom_file, tmp_dir)
        mlfsom_file = os.path.join(tmp_dir, 'mlfsom.tar.gz')
        assert os.path.exists(mlfsom_file), 'MLFSOM tarball not in temporary directory'

        # Untar the file
        t = tarfile.open(mlfsom_file, 'r:gz')
        try: t.extractall(tmp_dir)
        finally: t.close()

        # Delete unnecessary files
        for f in ftd_pre:
            fp = os.path.join(tmp_dir, f)
            if os.path.exists(fp): os.remove(fp)

        # Move the input files in
        shutil.copy(pdb_file, mlfsom_pdb)

        # Got to directory to run MLFSOM
        os.chdir(tmp_dir)

        # Convert the pdb_file to structure factors
        ano_sfall = CommandManager('./ano_sfall.com')
        ano_sfall.add_command_line_arguments(os.path.basename(mlfsom_pdb))
        ano_sfall.add_command_line_arguments('energy={}'.format(energy))
        ano_sfall.add_command_line_arguments('{}A'.format(res_h))
        if verbose: print('Running ./ano_sfall.com on {}').format(pdb_file)
        rc = ano_sfall.run()
        if rc != 0:
            print('STDOUT:')
            print(ano_sfall.output)
            print('STDERR:')
            print(ano_sfall.error)
        shutil.move('ideal_ano.mtz', mlfsom_mtz)
        with open(mlfsom_log, 'a') as fh:
            fh.write('==================================================>\n')
            fh.write('ano_sfall log\n')
            fh.write('==================================================>\n')
            fh.write(ano_sfall.output+'\n')
            fh.write('==================================================>\n')
            fh.write('ano_sfall errors\n')
            fh.write('==================================================>\n')
            fh.write(ano_sfall.error+'\n')
            fh.write('==================================================>\n')

        # Run MLFSOM on the resulting mtzfile
        mlfsom = CommandManager('./mlfsom.com')
        mlfsom.add_command_line_arguments(mlfsom_img)
        mlfsom.add_command_line_arguments('frames={}'.format(n_images))
        mlfsom.add_command_line_arguments('osc={}'.format(osc))
        if verbose: print('Running ./mlfsom.com on {}. This may take a while...').format(pdb_file)
        rc = mlfsom.run()
        if rc != 0:
            print('STDOUT:')
            print(mlfsom.output)
            print('STDERR:')
            print(mlfsom.error)
        with open(mlfsom_log, 'a') as fh:
            fh.write('==================================================>\n')
            fh.write('mlfsom log\n')
            fh.write('==================================================>\n')
            fh.write(mlfsom.output+'\n')
            fh.write('==================================================>\n')
            fh.write('mlfsom errors\n')
            fh.write('==================================================>\n')
            fh.write(mlfsom.error+'\n')
            fh.write('==================================================>\n')

    finally:
        # Change back to original directory
        os.chdir(cur_dir)
        # Delete temporary directory
        shutil.move(tmp_dir, out_dir)

    return ano_sfall, mlfsom




import jinja2
import urlparse, urllib

BAMBOO_HTML_ENV = jinja2.Environment(loader=jinja2.PackageLoader('bamboo', 'templates'))

def path2url(path):
    return urlparse.urljoin('file:', urllib.pathname2url(path))

def png2base64str(path):
    """Convert a png for embedding into an html page"""
    contents = open(path, 'rb').read().encode('base64').replace('\n', '')
    return contents



import os, sys, shutil, glob

#:::::::::::::::::::::::::::::::::#
# ############################### #
# ### Path Utility Functions  ### #
# ############################### #
#:::::::::::::::::::::::::::::::::#

def easy_directory(directory):
    """Checks a directory exists and creates it if not"""
    if not os.path.exists(directory):
        os.mkdir(directory)
    return directory

def rel_symlink(orig, link):
    """Make a relative symlink from link to orig"""
    assert os.path.exists(orig), 'FILE DOES NOT EXIST: {!s}'.format(orig)
    assert not os.path.exists(link), 'LINK ALREADY EXISTS: {!s}'.format(link)
    orig = os.path.abspath(orig)
    link = os.path.abspath(link)
    assert not link.endswith('/'), 'LINK CANNOT END WITH /'
    os.symlink(os.path.relpath(orig, start=os.path.dirname(link)), link)

def delete_with_glob(glob_str, verbose=True):
    for file in sorted(glob.glob(glob_str)):
        # I know that unlink is the same as remove...
        if os.path.islink(file):
            if verbose: print('Deleting link: {}'.format(file))
            os.unlink(file)
        elif os.path.isdir(file):
            pass
        else:
            if verbose: print('Deleting file: {}'.format(file))
            os.remove(file)

def delete_temporary_directory(tempdir):
    """Delete A Temporary Directory"""

    # Remove Temporary Files
    if tempdir.startswith('/tmp/'):
        shutil.rmtree(tempdir)
        return 0
    else:
        FlagError("Will Not Delete This Directory - Not in '/tmp' - Remove Manually : "+tempdir)
        return 1

def list_directory_contents(directory, templatename=None, templatestyle=None):
    """List Folders in a Directory"""

    # List the contents of the directory
    contents = os.listdir(directory)
    # Decide which are directories and which to keep
    if   templatename and templatestyle=='End':
        subdirs = [dir for dir in contents if os.isdir(os.path.join(directory,dir)) and dir.endswith(templatename)]
    elif templatename and templatestyle=='Start':
        subdirs = [dir for dir in contents if os.isdir(os.path.join(directory,dir)) and dir.startswith(templatename)]
    elif templatename and templatestyle=='Contains':
        subdirs = [dir for dir in contents if os.isdir(os.path.join(directory,dir)) and templatename in dir]
    else:
        subdirs = [dir for dir in contents if os.isdir(os.path.join(directory,dir))]
    # Construct Full Paths
    subdirs = [os.path.join(directory,dir) for dir in subdirs]

    return subdirs




import os, sys, copy

import libtbx.phil
from libtbx.utils import Sorry

def show_defaults_and_exit_maybe(master_phil, args):
    """Show master_phil and exit if requested"""

    # Show Defaults (just values)
    if '--show-defaults' in args:
        print('=========>\nShowing Default Parameters\n=========>')
        master_phil.show(expert_level=0, attributes_level=0)
    # Show Defaults (including help)
    elif '--help' in args:
        print('=========>\nShowing Default Parameters & Help\n=========>')
        master_phil.show(expert_level=0, attributes_level=1)
    # Show Defaults (including help and types)
    elif '--help-and-types' in args:
        print('=========>\nShowing Default Parameters & Help+Types\n=========>')
        master_phil.show(expert_level=0, attributes_level=2)
    # Show Defaults (everything)
    elif '--expert' in args:
        print('=========>\nShowing Default Parameters (Expert Level)\n=========>')
        master_phil.show(expert_level=3, attributes_level=3)
    else:
        return

    raise SystemExit('=========>\nNow Exiting.\n=========>')

def parse_phil_args(master_phil, args, blank_arg_prepend=None, home_scope=None):

    if blank_arg_prepend is None:
        pass
    elif isinstance(blank_arg_prepend, dict):
        for item in blank_arg_prepend.values():
            assert '=' in item
    elif isinstance(blank_arg_prepend, str):
        assert '=' in blank_arg_prepend
    else:
        raise Exception('blank_arg_prepend must be str or dict')

    # Copy the args so that we can remove items from the list without affecting args etc
    args = copy.copy(args)
    # Construct interpreter
    cmd_interpr = master_phil.command_line_argument_interpreter(home_scope=home_scope)

    # Process any args that are eff files
    eff_files = [f for f in args if ((f.endswith('.eff') or f.endswith('.def')) and (not f.count('=')) and os.path.isfile(f))]
    # Remove them from the original lists
    [args.remove(f) for f in eff_files]
    # Parse the 'eff' files - these should contain phils
    eff_sources = [libtbx.phil.parse(open(f, 'r').read()) for f in eff_files]

    # Process input arguments
    arg_sources = []
    for arg in args:
        try:
            # Prepend if blank
            if '=' not in arg:
                if isinstance(blank_arg_prepend, dict):
                    found_key = False
                    for key in blank_arg_prepend.keys():
                        if key is None:
                            continue
                        if arg.endswith(key):
                            arg = blank_arg_prepend[key]+arg
                            found_key = True
                            break
                    if (found_key == False) and (None in blank_arg_prepend.keys()):
                        arg = blank_arg_prepend[None]+arg
                elif isinstance(blank_arg_prepend, str):
                    arg = blank_arg_prepend+arg
            # Attempt to process arg
            cmd_line_args = cmd_interpr.process(arg=arg)
        except KeyboardInterrupt:
            raise
        except Exception:
            raise Sorry("Unknown file or keyword: %s" % arg)
        else:
            arg_sources.append(cmd_line_args)
    # Extract Scope object (putting eff sources first so that they're overridden if double-defined)
    working_phil = master_phil.fetch(sources=eff_sources+arg_sources)

    return working_phil

def extract_params_default(master_phil, args, blank_arg_prepend=None, home_scope=None):
    """Extract the parameters by a default script"""
    show_defaults_and_exit_maybe(master_phil, args)
    working_phil = parse_phil_args(master_phil, args, blank_arg_prepend=blank_arg_prepend, home_scope=home_scope)
    return working_phil

def run_default(run, master_phil, args, blank_arg_prepend=None):
    """Run a program via a standard setup of functions and objects"""
    working_phil = extract_params_default(master_phil=master_phil, args=args, blank_arg_prepend=blank_arg_prepend)
    out = run(params=working_phil.extract())
    return 0



from matplotlib import pyplot
pyplot.style.use('ggplot')

import numpy

class Radar(object):

    def __init__(self, titles, rect=None, fig=None):
        # Make a rectangle
        if rect is None: rect = [0.05, 0.05, 0.95, 0.95]
        # Number of plots
        self.n = len(titles)
        # Number of data points
        self.d = 0

        # Control flags
        self._plotted    = False

        # Inside and Outside offsets
        self._fixed_offset = 0.2
        self._fixed_buffer = 0.2

        # Axis limits + ranges + ticks
        self._limits = []
        self._ranges = []
        self._ticks  = []
        # Scaling to 0-1 (Scale all to this scale)
        self._scales = []
        self._offset = []
        # Data Points
        self._data = []
        self._args = []
        self._kwgs = []
        # Invert Axes?
        self._invert = [0]*self.n

        # Create a figure if none given
        if fig: self.fig = fig
        else:   self.fig = pyplot.figure()

        # Store titles
        self.titles = titles
        # Create and store angles and axes
        self.angles = [a if a <=360. else a - 360. for a in numpy.arange(90, 90+360, 360.0/self.n)]
        self.axes   = [self.fig.add_axes(rect, projection="polar", label="axes%d" % i) for i in range(self.n)]

        # Get main axis, add theta grids
        self.ax = self.axes[0]
        self.ax.set_thetagrids(self.angles, labels=titles, frac=1.2, fontsize=14, weight="bold", color="black")
        # Remove all things from other axes
        for ax in self.axes[1:]:
            ax.patch.set_visible(False)
            ax.grid("off")
            ax.xaxis.set_visible(False)
            self.ax.yaxis.grid(False)

    def _normalise(self, vals):
        """Normalise values to the scaled axis"""
        assert len(vals) == self.n
        ret_vals = []
        for vls,s,o,i in zip(vals, self._scales, self._offset, self._invert):
            ret_vals_sub = []
            for v in vls:
                if (v is None) or numpy.isnan(v): ret_vals_sub.append(self._fixed_offset+0.5)
                else:                             ret_vals_sub.append(self._fixed_offset+(1.0-i)*(v*s+o)+i*(1.0-(v*s+o)))
            ret_vals.append(ret_vals_sub)
        return ret_vals

    def _default_limits(self):
        """Set the limits to the range of the data"""
        limits = []
        for d in numpy.array(self._data).T:
            if   len(d)==1:      limits.append((d[0]-1,d[0]+1))
            elif min(d)==max(d): limits.append((d[0]-1,d[0]+1))
            else:                limits.append((min(d),max(d)))
        self.set_limits(limits)

    def _calculate_scales(self):
        """Calculate the scales to normalise the axis limits to {0,1}"""
        # Calculate ranges for each axis
        self._ranges = [1.0*(max-min) for (min, max) in self._limits]
        # Scale this axis to the reference
        self._scales = [1.0/self._ranges[i] for i in range(self.n)]
        self._offset = [-min*sca for (min,max), sca in zip(self._limits, self._scales)]

    def _default_ticks(self):
        """Default ticks for added values"""
        ticks = [[min(d),max(d)] for d in numpy.array(self._data).T]
        self.set_ticks(ticks)

    def set_limits(self, limits):
        """Store limits"""
        assert len(limits) == self.n
        self._limits = limits

    def set_ticks(self, values, labels=None):
        """Store ticks"""
        if labels is None: labels=values
        assert len(values) == self.n
        assert len(labels) == self.n
        self._ticks = (values, labels)

    def set_inversion(self, bool):
        """Select which axes whould be inverted"""
        assert len(bool) == self.n
        self._invert = bool

    def _apply_limits(self):
        """Scale the axes to {0,1}"""
        norm_limits = self._normalise(self._limits)
        for ax, (min, max) in zip(self.axes, norm_limits):
            ax.set_ylim(0.0, 1.0+self._fixed_offset+self._fixed_buffer)

    def _filter_values_and_labels(self, values, labels, filter_large=True, filter_small=True):
        """Filter values and labels - apply floors and ceilings to allowed values"""
        filt_vals = []; filt_labs = []
        filt_upper = self._fixed_offset + 1.0
        filt_lower = self._fixed_offset + 0.0
        for vals, labs in zip(values, labels):
            vs=[]; ls=[];
            for v, l in zip(vals, labs):
                if (v>filt_lower or not filter_small) and (v<filt_upper or not filter_large):
                    vs.append(v); ls.append(l)
                elif v>=filt_upper:
                    vs.append(filt_upper); ls.append(l)
                elif v<=filt_lower:
                    vs.append(filt_lower); ls.append(l)
            filt_vals.append(vs); filt_labs.append(ls)
        return (filt_vals, filt_labs)

    def _apply_ticks(self):
        tick_vals = self._normalise(self._ticks[0])
        tick_labs = self._ticks[1]
        tick_vals, tick_labs = self._filter_values_and_labels(tick_vals, tick_labs, filter_large=True, filter_small=True)
        for ax, angle, vals, labs in zip(self.axes, self.angles, tick_vals, tick_labs):
            ax.set_rgrids(vals, labels=labs, angle=angle, fontsize=14, weight="bold", color="black")
            ax.spines["polar"].set_visible(False)
            ax.xaxis.grid(True,color='black',linestyle='-')

    def add(self, values, *args, **kwgs):
        assert len(values) == self.n
        self._data.append(values)
        self._args.append(args)
        self._kwgs.append(kwgs)
        self.d = len(self._data)

    def show(self):
        if not self._plotted: self.plot()
        self.fig.show()

    def plot(self):
        if not self._limits: self._default_limits()
        if not self._scales: self._calculate_scales()
        if not self._ticks:  self._default_ticks()
        self._apply_limits()
        self._apply_ticks()
        self._plotted = True
        norm_vals = numpy.array(self._normalise(numpy.array(self._data).T.tolist())).T.tolist()
        filt_vals, dummy = self._filter_values_and_labels(norm_vals, norm_vals, filter_large=False, filter_small=True)
        for values, args, kwgs in zip(filt_vals, self._args, self._kwgs):
            angle = numpy.deg2rad(numpy.r_[self.angles, self.angles[0]])
            vals = numpy.r_[values, values[0]]
            self.ax.plot(angle, vals, *args, **kwgs)

    def savefig(self, filename):
        if not self._plotted: self.plot()
        self.fig.savefig(filename, bbox_inches="tight", pad_inches=0.5)

    def close(self):
        pyplot.close(self.fig)



import iotbx.pdb

from scitbx.math import basic_statistics

from bamboo.edstats import Edstats
from giant.structure.b_factors import BfactorStatistics, normalise_b_factors_to_z_scores
from giant.structure.html import write_html_summary

class StructureSummary(object):

    def __init__(self, pdb_input=None, pdb_hierarchy=None, pdb_file=None, mtz_file=None):
        """Summarise some characteristics of a structure"""

        assert [pdb_file, pdb_input, pdb_hierarchy].count(None)==2,'Provide pdb_file or pdb_input OR pdb_hierarchy'
        if pdb_file:  pdb_input     = iotbx.pdb.input(file_name = pdb_file)
        if pdb_input: pdb_hierarchy = pdb_input.construct_hierarchy()
        # Structure + Data
        self.mtz_file       = mtz_file
        self.pdb_input      = pdb_input
        self.pdb_hierarchy  = pdb_hierarchy
        # B-factors
        self.b_factors = BfactorStatistics.from_pdb(pdb_hierarchy=pdb_hierarchy)
        # Edstats
        if mtz_file: self.edstats = Edstats(mtz_file=mtz_file, pdb_file=pdb_file)
        else:        self.edstats = None

    def normalise_b_factors(self, root=None):
        if root is None: root = self.pdb_hierarchy
        return normalise_b_factors_to_z_scores(pdb_hierarchy=root, b_factor_summary=self.b_factors)

    def _get_atom_group_edstats_scores(self, atom_group):
        resname = atom_group.resname
        chain   = atom_group.parent().parent().id
        res_i   = atom_group.parent().resseq_as_int()
        inscode = ' '
        return self.edstats.scores[(resname, chain, res_i, inscode)]

    def get_atom_group_summary(self, atom_group):
        return atomGroupSummary( atom_group = atom_group,
                                 global_b_factor_statistics = self.b_factors )
    def get_edstats_scores(self, atom_group):
        return self._get_atom_group_edstats_scores(atom_group=atom_group)

    def to_html(self, fname):
        write_html_summary(fname=fname, atom_group_summaries=[self.get_atom_group_summary(ag) for ag in self.pdb_hierarchy.atom_groups()])

class AtomGroupSummary(object):

    def __init__(self, atom_group, global_b_facs_stats=None):

        self.atom_group = atom_group

        # Meta
        self.residue_class = iotbx.pdb.common_residue_names_get_class(atom_group.resname)

        # B-Factors
        self.b_facs         = atom_group.atoms().extract_b()
        self.b_facs_stats   = basic_statistics(atom_group.atoms().extract_b())
        # B-Factors Z-Statistics
        if global_b_facs_stats is not None:
            self.b_facs_z       = global_b_facs_stats.to_z_score(b_vals=atom_group.atoms().extract_b())
            self.b_facs_z_stats = basic_statistics(self.b_factors_z)
        else:
            self.b_facs_z       = None
            self.b_facs_z_stats = None

        # Occupancies
        self.occies         = atom_group.atoms().extract_occ()
        self.occies_stats   = basic_statistics(self.occies)

        # Coordinates
        self.centroid       = atom_group.atoms().extract_xyz().mean()

    def to_html(self, fname):
        write_html_summary(fname=fname, atom_group_summaries=[self])




import os

from bamboo.wrappers.ligand_builders import BuilderObject

class ElbowObject(BuilderObject):

    def _custom_init(self):
        self.program = 'phenix.elbow'
        self.name = 'elbow'

    def _create_program_arguments(self, ligsmile, outdir, outfile, flags):
        """Prepare the arguments for Elbow to build the ligand"""

        # Format Settings
        cmd_line_flags, std_inpt_flags = self._convert_flags(flags)
        # Form Command Line Args
        cmd_line_args = ['--smiles',ligsmile,'--output',self.outtemplate,'--name',self.ligname,'--id',self.ligname]
        cmd_line_args.extend(cmd_line_flags)
        # Form Std Input
        std_inpt_args = []
        std_inpt_args.extend(std_inpt_flags)

        return cmd_line_args, std_inpt_args

    def _convert_flags(self, flags):
        """Take the flags passed and generate the command string appropriately"""

        cmd_line_flags = []
        std_inpt_flags = []

        if 'addHydrogens' in flags:
            cmd_line_flags.extend(['--add-hydrogens','True'])
        else:
            cmd_line_flags.extend(['--add-hydrogens','False'])

        return cmd_line_flags, std_inpt_flags



import os,sys,re

from bamboo.common.file import FileObj
from bamboo.common.command import CommandManager

################################################################################################################
### CLASSES                                                                                                    #
################################################################################################################

class MtzMeta(object):
    """Object to hold the meta data for an MTZ file"""

    def __init__(self):
        self.reslow = None
        self.reshigh = None
        self.spacegroup = None
        self.spacegroupno = None
        self.cell = None
        self.cryst = None
    def __str__(self):
        return str(self.__dict__)

class ColumnLabels(object):
    """Object to contain the column labels of an MTZ file"""

    def __init__(self):
        self.f = None
        self.sigf = None
        self.phas = None
        self.free = None
        self.comp_f = None
        self.comp_p = None
        self.diff_f = None
        self.diff_p = None
    def __str__(self):
        return str(self.__dict__)

class MtzSummary(object):
    """Class for summarising an MTZ file"""

    def __init__(self, mtz_file):
        self.path = os.path.abspath(mtz_file)
        self.file = FileObj(mtz_file)
        # Read the mtz_file and create summary
        self.summary = get_mtz_summary_dict(mtz_file)
        # Record information from summary
        self.label = ColumnLabels()
        self.label.f    = self.summary['f_labels'][0]    if self.summary['f_labels']    else None
        self.label.sigf = self.summary['sigf_labels'][0] if self.summary['sigf_labels'] else None
        self.label.i    = self.summary['i_labels'][0]    if self.summary['i_labels']    else None
        self.label.sigi = self.summary['sigi_labels'][0] if self.summary['sigi_labels'] else None
        self.label.phas = self.summary['p_labels'][0]    if self.summary['p_labels']    else None
        self.label.free = self.summary['r_flags'][0]     if self.summary['r_flags']     else None
        self.label.comp_f = self.summary['wtmap_f_comp'][0] if self.summary['wtmap_f_comp'] else None
        self.label.comp_p = self.summary['wtmap_p_comp'][0] if self.summary['wtmap_p_comp'] else None
        self.label.diff_f = self.summary['wtmap_f_diff'][0] if self.summary['wtmap_f_diff'] else None
        self.label.diff_p = self.summary['wtmap_p_diff'][0] if self.summary['wtmap_p_diff'] else None
        # Get some Meta
        self.data = MtzMeta()
        self.data.reslow        = self.summary['reslow']        if self.summary['reslow']       else None
        self.data.reshigh       = self.summary['reshigh']       if self.summary['reshigh']      else None
        self.data.spacegroup    = self.summary['spacegroup']    if self.summary['spacegroup']   else None
        self.data.spacegroupno  = self.summary['spacegroupno']  if self.summary['spacegroupno'] else None
        self.data.cell          = self.summary['cell']
        self.data.cryst         = dict(zip(['A','B','C','ALPHA','BETA','GAMMA','SG'],self.data.cell+[self.data.spacegroup]))
        self.data.a,     self.data.b,    self.data.c     = self.summary['cell'][0:3]
        self.data.alpha, self.data.beta, self.data.gamma = self.summary['cell'][3:6]

################################################################################################################
### FUNCTIONS                                                                                                  #
################################################################################################################

def get_mtz_summary_dict(mtz_file):
    """Get an MTZ Summary"""

    # Extract Contents of MTZ
    MTZDMP = CommandManager('mtzdmp')
    MTZDMP.add_command_line_arguments(mtz_file)
    MTZDMP.run()
    # Check for errors
    if MTZDMP.process.returncode != 0:
        raise RuntimeError('mtzdmp failed to read file {!s}:\nReturn: {!s}\nOut: {!s}'.format(mtz_file,MTZDMP.process.returncode,MTZDMP.output))

    # Create empty dict to contain the summary
    summary = {}

    # Get the resolution range
    regex = re.compile('\*  Resolution Range :.*\n.*\n.*\((.*)A \)\n')
    matches = regex.findall(MTZDMP.output)
    assert matches, 'No Resolution Range found in MTZFile {!s}'.format(mtz_file)
    assert len(matches)==1, 'Too many matching lines found for Resolution Range in MTZFile {!s}\n\t{!s}'.format(mtz_file,matches)
    summary['reslow'],summary['reshigh'] = map(float,matches[0].replace(' ','').split('-'))

    # Get the Number of Columns
    regex = re.compile('\* Number of Columns =(.*)\n')
    matches = regex.findall(MTZDMP.output)
    assert matches, 'No Number of Columns found for {!s}'.format(mtz_file)
    assert len(matches)==1, 'Too many matching lines found for Number of Columns in MTZFile {!s}\n\t{!s}'.format(mtz_file,matches)
    summary['numcols'] = int(matches[0].strip())

    # Get the Number of Reflections
    regex = re.compile('\* Number of Reflections =(.*)\n')
    matches = regex.findall(MTZDMP.output)
    assert matches, 'No Number of Reflections found for {!s}'.format(mtz_file)
    assert len(matches)==1, 'Too many matching lines found for Number of Reflections in MTZFile {!s}\n\t{!s}'.format(mtz_file,matches)
    summary['numreflections'] = int(matches[0].strip())

    # Get the Column Labels
    regex = re.compile('\* Column Labels :.*\n.*\n(.*)\n')
    matches = regex.findall(MTZDMP.output)
    assert matches, 'No Column Labels found for {!s}'.format(mtz_file)
    assert len(matches)==1, 'Too many matching lines found for Column Headings in MTZFile {!s}\n\t{!s}'.format(mtz_file,matches)
    summary['colheadings'] = matches[0].strip().split()

    # Get the Column Types
    regex = re.compile('\* Column Types :.*\n.*\n(.*)\n')
    matches = regex.findall(MTZDMP.output)
    assert matches, 'No Column Types found for {!s}'.format(mtz_file)
    assert len(matches)==1, 'Too many matching lines found for Column Types in MTZFile {!s}\n\t{!s}'.format(mtz_file,matches)
    summary['coltypes'] = matches[0].strip().split()

    # Get the different datasets
    regex = re.compile('\* Associated datasets :.*\n.*\n(.*)\n')
    matches = regex.findall(MTZDMP.output)
    assert matches, 'No Dataset Numbers found for {!s}'.format(mtz_file)
    assert len(matches)==1, 'Too many matching lines found for Dataset Numbers in MTZFile {!s}\n\t{!s}'.format(mtz_file,matches)
    summary['coldatasets'] = matches[0].strip().split()

    # Get the Spacegroup
    regex = re.compile('\* Space group =.*\'(.*)\'.\(number(.*)\)')
    matches = regex.findall(MTZDMP.output)
    assert matches, 'No Space Group found for {!s}'.format(mtz_file)
    assert len(matches)==1, 'Too many matching lines found for Spacegroup in MTZFile {!s}\n\t{!s}'.format(mtz_file,matches)
    summary['spacegroup'] = matches[0][0].strip()
    summary['spacegroupno'] = int(matches[0][1].strip())

    # Get the Cell Dimensions
    regex = re.compile('\* Cell Dimensions :.*\n.*\n(.*)\n')
    matches = regex.findall(MTZDMP.output)
    assert matches, 'No Cell Dimensions found for {!s}'.format(mtz_file)
    assert len(matches)==1, 'Too many matching lines found for Cell Dimensions in MTZFile {!s}\n\t{!s}'.format(mtz_file,matches)
    summary['cell'] = map(float,matches[0].split())

    # Get the Cell Dimensions

    # Sort the flags
    # F: Amplitudes, H: Miller Indices, I: Integers, P: Phases, Q: Standard Deviations, J: Intensities
    sfac = [summary['colheadings'][i] for i,type in enumerate(summary['coltypes']) if type=='F']
    inty = [summary['colheadings'][i] for i,type in enumerate(summary['coltypes']) if type=='J']
    mill = [summary['colheadings'][i] for i,type in enumerate(summary['coltypes']) if type=='H']
    ints = [summary['colheadings'][i] for i,type in enumerate(summary['coltypes']) if type=='I']
    phas = [summary['colheadings'][i] for i,type in enumerate(summary['coltypes']) if type=='P']
    sdev = [summary['colheadings'][i] for i,type in enumerate(summary['coltypes']) if type=='Q']

    # 2FOFC cols
    wt_f_map_opts = ['2FOFCWT','FWT']
    wt_p_map_opts = ['PH2FOFCWT','PHWT','PHFWT']
    # FOFC cols
    wt_f_dif_opts = ['FOFCWT','DELFWT']
    wt_p_dif_opts = ['PHFOFCWT','DELPHWT','PHDELWT']
    # Record 2FOFC pair (composite map)
    summary['wtmap_f_comp'] = [s for s in sfac if s in wt_f_map_opts]
    summary['wtmap_p_comp'] = [p for p in phas if p in wt_p_map_opts]
    # Record FOFC pair (different map)
    summary['wtmap_f_diff'] = [s for s in sfac if s in wt_f_dif_opts]
    summary['wtmap_p_diff'] = [p for p in phas if p in wt_p_dif_opts]

    # Find the main F col
    summary['f_labels'] = [s for s in sfac if (s in ['F','FP','FCTR','FOSC','FOBS'] \
                            or s.startswith('F_') or s.startswith('FP_') \
                            or s.upper().startswith('F-OBS') or s.upper().startswith('FOUT_'))]
    if not summary['f_labels']:
        summary['f_labels'] = [s for s in sfac if s.startswith('FP')]
    # Find the main SIGF col
    summary['sigf_labels'] = [s for s in sdev if (s in ['SIGF','SIGFP','SIGFOBS'] \
                            or s.startswith('SIGF_') or s.startswith('SIGFP_') \
                            or s.upper().startswith('SIGF-OBS') or s.upper().startswith('SIGFOUT_'))]
    if not summary['sigf_labels']:
        summary['sigf_labels'] = [s for s in sdev if s.startswith('SIGFP')]
    # Find the I cols
    summary['i_labels'] = [s for s in inty if s.startswith('I')]
    # Find the SIGI cols
    summary['sigi_labels'] = [s for s in sdev if s.startswith('SIGI')]
    # Find the F_calc
    summary['f_calcs'] = [s for s in sfac if (s=='FC' or s=='FMODEL' or s.startswith('FC_') or s.upper().startswith('F-MODEL'))]
    # Find the PHI_calcs
    summary['p_calcs'] = [p for p in phas if (p=='PHIC' or p=='PHIFMODEL' or p.startswith('PHIC_') or p.upper().startswith('PHIF-MODEL'))]
    # Find the main phase col
    summary['p_labels'] = summary['p_calcs']
    # Find the RFree Flag
    summary['r_flags'] = [r for r in ints if ('RFREE' in r.upper() or 'FREER' in r.upper() or r=='FREE' or r.upper().startswith('R-FREE'))]

    # XXX Will probably delete this later
    summary['unknown'] = [c for c in summary['colheadings'] if c not in summary['f_labels']+summary['sigf_labels']+summary['f_calcs']+summary['p_calcs']+summary['p_labels']+summary['r_flags']+summary['wtmap_f_comp']+summary['wtmap_p_comp']+summary['wtmap_f_diff']+summary['wtmap_p_diff']]

    return summary

def get_mtz_resolution(mtz_file):
    """Gets the max resolution from the file"""

    # Extract Contents of MTZ
    MTZDMP = CommandManager('mtzdmp')
    MTZDMP.add_command_line_arguments(mtz_file)
    MTZDMP.run()
    # Check for errors
    if MTZDMP.process.returncode != 0:
        raise RuntimeError('mtzdmp failed to read file {!s}:\nReturn: {!s}\nOut: {!s}'.format(mtz_file,MTZDMP.process.returncode,MTZDMP.output))
    # Search for the Column Headings
    regex = re.compile('\*  Resolution Range :.*\n.*\n.*\((.*)A \)\n')
    matches = regex.findall(MTZDMP.output)
    # Check for validity of matches
    assert matches, 'No Resolution Range found in MTZFile {!s}'.format(mtz_file)
    assert len(matches)==1, 'Too many matching lines found for Column Headings in MTZFile {!s}\n\t{!s}'.format(mtz_file,matches)
    # Return
    return map(float,matches[0].replace(' ','').split('-'))

